/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_tanukisoftware_wrapper_WrapperManager */

#ifndef _Included_org_tanukisoftware_wrapper_WrapperManager
#define _Included_org_tanukisoftware_wrapper_WrapperManager
#ifdef __cplusplus
extern "C" {
#endif
#undef org_tanukisoftware_wrapper_WrapperManager_DEFAULT_PORT
#define org_tanukisoftware_wrapper_WrapperManager_DEFAULT_PORT 15003L
#undef org_tanukisoftware_wrapper_WrapperManager_DEFAULT_CPU_TIMEOUT
#define org_tanukisoftware_wrapper_WrapperManager_DEFAULT_CPU_TIMEOUT 10000L
#undef org_tanukisoftware_wrapper_WrapperManager_TICK_MS
#define org_tanukisoftware_wrapper_WrapperManager_TICK_MS 100L
#undef org_tanukisoftware_wrapper_WrapperManager_TIMER_FAST_THRESHOLD
#define org_tanukisoftware_wrapper_WrapperManager_TIMER_FAST_THRESHOLD 1728000L
#undef org_tanukisoftware_wrapper_WrapperManager_TIMER_SLOW_THRESHOLD
#define org_tanukisoftware_wrapper_WrapperManager_TIMER_SLOW_THRESHOLD 1728000L
#undef org_tanukisoftware_wrapper_WrapperManager_BACKEND_TYPE_UNKNOWN
#define org_tanukisoftware_wrapper_WrapperManager_BACKEND_TYPE_UNKNOWN 0L
#undef org_tanukisoftware_wrapper_WrapperManager_BACKEND_TYPE_SOCKET_V4
#define org_tanukisoftware_wrapper_WrapperManager_BACKEND_TYPE_SOCKET_V4 1L
#undef org_tanukisoftware_wrapper_WrapperManager_BACKEND_TYPE_SOCKET_V6
#define org_tanukisoftware_wrapper_WrapperManager_BACKEND_TYPE_SOCKET_V6 2L
#undef org_tanukisoftware_wrapper_WrapperManager_BACKEND_TYPE_PIPE
#define org_tanukisoftware_wrapper_WrapperManager_BACKEND_TYPE_PIPE 4L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_START
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_START 100L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_STOP
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_STOP 101L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_RESTART
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_RESTART 102L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_PING
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_PING 103L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_STOP_PENDING
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_STOP_PENDING 104L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_START_PENDING
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_START_PENDING 105L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_STARTED
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_STARTED 106L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_STOPPED
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_STOPPED 107L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_KEY
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_KEY 110L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_BADKEY
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_BADKEY 111L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_LOW_LOG_LEVEL
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_LOW_LOG_LEVEL 112L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_PING_TIMEOUT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_PING_TIMEOUT 113L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_SERVICE_CONTROL_CODE
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_SERVICE_CONTROL_CODE 114L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_PROPERTIES
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_PROPERTIES 115L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_LOG
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_LOG 116L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_CHILD_LAUNCH
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_CHILD_LAUNCH -124L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_CHILD_TERM
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_CHILD_TERM -123L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_LOGFILE
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_LOGFILE -122L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_CHECK_DEADLOCK
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_CHECK_DEADLOCK -121L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_DEADLOCK
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_DEADLOCK -120L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_APPEAR_ORPHAN
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_APPEAR_ORPHAN -119L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_PAUSE
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_PAUSE -118L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_RESUME
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_RESUME -117L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_GC
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_GC -116L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_FIRE_USER_EVENT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_FIRE_USER_EVENT -115L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_SECOND_INVOCATION_EVENT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_MSG_SECOND_INVOCATION_EVENT -114L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_C_EVENT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_C_EVENT 200L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_CLOSE_EVENT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_CLOSE_EVENT 201L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_LOGOFF_EVENT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_LOGOFF_EVENT 202L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_SHUTDOWN_EVENT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_SHUTDOWN_EVENT 203L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_TERM_EVENT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_TERM_EVENT 204L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_HUP_EVENT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_HUP_EVENT 205L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_USR1_EVENT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_USR1_EVENT 206L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_USR2_EVENT
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_CTRL_USR2_EVENT 207L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_DEBUG
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_DEBUG 1L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_INFO
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_INFO 2L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_STATUS
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_STATUS 3L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_WARN
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_WARN 4L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_ERROR
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_ERROR 5L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_FATAL
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_FATAL 6L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_ADVICE
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_ADVICE 7L
#undef org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_NOTICE
#define org_tanukisoftware_wrapper_WrapperManager_WRAPPER_LOG_LEVEL_NOTICE 8L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_START
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_START 65536L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_STOP
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_STOP 1L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_PAUSE
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_PAUSE 2L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_CONTINUE
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_CONTINUE 3L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_INTERROGATE
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_INTERROGATE 4L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_SHUTDOWN
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_SHUTDOWN 5L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_QUERYSUSPEND
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_QUERYSUSPEND 3328L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_QUERYSUSPENDFAILED
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_QUERYSUSPENDFAILED 3330L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_SUSPEND
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_SUSPEND 3332L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_RESUMECRITICAL
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_RESUMECRITICAL 3334L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_RESUMESUSPEND
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_RESUMESUSPEND 3335L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_BATTERYLOW
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_BATTERYLOW 3337L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_POWERSTATUSCHANGE
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_POWERSTATUSCHANGE 3338L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_OEMEVENT
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_OEMEVENT 3339L
#undef org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_RESUMEAUTOMATIC
#define org_tanukisoftware_wrapper_WrapperManager_SERVICE_CONTROL_CODE_POWEREVENT_RESUMEAUTOMATIC 3346L
/* Inaccessible static: m_out */
/* Inaccessible static: m_err */
/* Inaccessible static: m_outInfo */
/* Inaccessible static: m_outError */
/* Inaccessible static: m_outDebug */
/* Inaccessible static: m_windows */
/* Inaccessible static: m_macosx */
/* Inaccessible static: m_aix */
/* Inaccessible static: m_securityManagerChecked */
/* Inaccessible static: m_disposed */
/* Inaccessible static: m_starting */
/* Inaccessible static: m_started */
/* Inaccessible static: m_instance */
/* Inaccessible static: m_hook */
/* Inaccessible static: m_hookTriggered */
/* Inaccessible static: m_hookRemoveFailed */
/* Inaccessible static: m_shutdownJVMComplete */
/* Inaccessible static: m_shutdownLockMap */
/* Inaccessible static: m_shutdownLocks */
/* Inaccessible static: m_runningExecs */
/* Inaccessible static: m_args */
/* Inaccessible static: m_backendType */
/* Inaccessible static: m_backendConnected */
/* Inaccessible static: m_backendOS */
/* Inaccessible static: m_backendIS */
/* Inaccessible static: m_port */
/* Inaccessible static: m_jvmPort */
/* Inaccessible static: m_jvmPortMin */
/* Inaccessible static: m_jvmPortMax */
/* Inaccessible static: m_wrapperPortAddress */
/* Inaccessible static: m_key */
/* Inaccessible static: m_soTimeout */
/* Inaccessible static: m_cpuTimeout */
/* Inaccessible static: m_startedTicks */
/* Inaccessible static: m_lowLogLevel */
/* Inaccessible static: m_ignoreSignals */
/* Inaccessible static: m_detachStarted */
/* Inaccessible static: m_commRunner */
/* Inaccessible static: m_commRunnerStarted */
/* Inaccessible static: m_eventRunner */
/* Inaccessible static: m_eventRunnerTicks */
/* Inaccessible static: m_startupRunner */
/* Inaccessible static: m_useSystemTime */
/* Inaccessible static: m_timerFastThreshold */
/* Inaccessible static: m_timerSlowThreshold */
/* Inaccessible static: m_disableTests */
/* Inaccessible static: m_listenerForceStop */
/* Inaccessible static: m_jvmBits */
/* Inaccessible static: m_ticks */
/* Inaccessible static: m_listener */
/* Inaccessible static: m_lastPingTicks */
/* Inaccessible static: m_backendSocket */
/* Inaccessible static: m_appearHung */
/* Inaccessible static: m_slowSeconds */
/* Inaccessible static: m_ignoreUserLogoffs */
/* Inaccessible static: m_service */
/* Inaccessible static: m_debug */
/* Inaccessible static: m_logFinalizer */
/* Inaccessible static: m_jvmId */
/* Inaccessible static: m_stoppingInit */
/* Inaccessible static: m_stopping */
/* Inaccessible static: m_stoppingThread */
/* Inaccessible static: m_stopped */
/* Inaccessible static: m_pendingStopMessage */
/* Inaccessible static: m_exitCode */
/* Inaccessible static: m_libraryOK */
/* Inaccessible static: m_commandBuffer */
/* Inaccessible static: m_logFile */
/* Inaccessible static: m_properties */
/* Inaccessible static: m_wrapperEventListenerMaskList */
/* Inaccessible static: m_wrapperEventListenerMasks */
/* Inaccessible static: m_produceCoreEvents */
/* Inaccessible static: m_res */
/* Inaccessible static: m_professionalEdition */
/* Inaccessible static: m_standardEdition */
/* Inaccessible static: PROPERTY_SEPARATOR */
/* Inaccessible static: m_backendReadBuffer */
/* Inaccessible static: class_00024org_00024tanukisoftware_00024wrapper_00024WrapperManager */
/* Inaccessible static: class_00024java_00024lang_00024String */
/* Inaccessible static: class_00024java_00024lang_00024Object */
/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeInit
 * Signature: (Z)V
 */
JNIEXPORT void JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit
  (JNIEnv *, jclass, jboolean);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeGetLibraryVersion
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetLibraryVersion
  (JNIEnv *, jclass);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeGetJavaPID
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetJavaPID
  (JNIEnv *, jclass);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeIsProfessionalEdition
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeIsProfessionalEdition
  (JNIEnv *, jclass);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeIsStandardEdition
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeIsStandardEdition
  (JNIEnv *, jclass);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeGetControlEvent
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetControlEvent
  (JNIEnv *, jclass);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeRedirectPipes
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeRedirectPipes
  (JNIEnv *, jclass);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeRequestThreadDump
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeRequestThreadDump
  (JNIEnv *, jclass);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    accessViolationInner
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_accessViolationInner
  (JNIEnv *, jclass);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeSetConsoleTitle
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeSetConsoleTitle
  (JNIEnv *, jclass, jstring);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeGetUser
 * Signature: (Z)Lorg/tanukisoftware/wrapper/WrapperUser;
 */
JNIEXPORT jobject JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetUser
  (JNIEnv *, jclass, jboolean);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeGetInteractiveUser
 * Signature: (Z)Lorg/tanukisoftware/wrapper/WrapperUser;
 */
JNIEXPORT jobject JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetInteractiveUser
  (JNIEnv *, jclass, jboolean);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeListServices
 * Signature: ()[Lorg/tanukisoftware/wrapper/WrapperWin32Service;
 */
JNIEXPORT jobjectArray JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeListServices
  (JNIEnv *, jclass);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeSendServiceControlCode
 * Signature: (Ljava/lang/String;I)Lorg/tanukisoftware/wrapper/WrapperWin32Service;
 */
JNIEXPORT jobject JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeSendServiceControlCode
  (JNIEnv *, jclass, jstring, jint);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeExec
 * Signature: ([Ljava/lang/String;Ljava/lang/String;Lorg/tanukisoftware/wrapper/WrapperProcessConfig;Z)Lorg/tanukisoftware/wrapper/WrapperProcess;
 */
JNIEXPORT jobject JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeExec
  (JNIEnv *, jclass, jobjectArray, jstring, jobject, jboolean);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeWrapperGetEnv
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeWrapperGetEnv
  (JNIEnv *, jclass, jstring);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeLoadWrapperResources
 * Signature: (Ljava/lang/String;Ljava/lang/String;Z)Lorg/tanukisoftware/wrapper/WrapperResources;
 */
JNIEXPORT jobject JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeLoadWrapperResources
  (JNIEnv *, jclass, jstring, jstring, jboolean);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeCheckDeadLocks
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeCheckDeadLocks
  (JNIEnv *, jclass);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeGetPortStatus
 * Signature: (ILjava/lang/String;I)I
 */
JNIEXPORT jint JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetPortStatus
  (JNIEnv *, jclass, jint, jstring, jint);

/*
 * Class:     org_tanukisoftware_wrapper_WrapperManager
 * Method:    nativeGetDpiScale
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetDpiScale
  (JNIEnv *, jclass);

#ifdef __cplusplus
}
#endif
#endif
