/*
 * Copyright (c) 1999, 2016 Tanuki Software, Ltd.
 * http://www.tanukisoftware.com
 * All rights reserved.
 *
 * This software is the proprietary information of Tanuki Software.
 * You shall use it only in accordance with the terms of the
 * license agreement you entered into with Tanuki Software.
 * http://wrapper.tanukisoftware.com/doc/english/licenseOverview.html
 */

#ifndef _TESTSUITE_H
#define _TESTSUITE_H

#include "CUnit/Basic.h"
#include "wrapper_i18n.h"

extern int tsEX_suiteExample();
extern int tsFLTR_suiteFilter();
extern int tsJAP_suiteJavaAdditionalParam();
extern int tsHASH_suiteHashMap();

#endif
